#include<avr/io.h>
#include <util/delay.h>
#include <stdlib.h> 
#include <stdio.h> 
#include "bmp_pixel.h"
#include "Nokia5110.h"


// Defines 
#define LCD_WIDTH  84
#define LCD_HEIGHT 48
#define INITIAL_SIZE 3
#define MAX_SIZE     10
#define BLOCK_SIZE 4
#define UP    0
#define DOWN  1
#define LEFT  2
#define RIGHT 3
#define NONE 4


// Struct that defines a point on the map
typedef struct {
    uint8_t x;
    uint8_t y;
} coord;


coord snake[MAX_SIZE];
uint32_t snake_direction;
coord food;

// Snake's head, tail, and size attributes
uint32_t head_position, tail_position, snake_size;

// Initialize the 4 buttons, used for directions
void init_buttons(void) {

    DDRB &= ~(1 << PB1);
	DDRB &= ~(1 << PB2);
	DDRB &= ~(1 << PB3);
	DDRB &= ~(1 << PB4);


	PORTB |= (1 << PB1);
	PORTB |= (1 << PB2);
	PORTB |= (1 << PB3);
	PORTB |= (1 << PB4);
}


// Check pressed buttons
void checkInput(void) {

	if ((PINB & (1 << PB1)) == 0) {
		//PORTD |= (1 << PD7);
		snake_direction = RIGHT;
	}
	if ((PINB & (1 << PB2)) == 0) {
		snake_direction = UP;
	}
	if ((PINB & (1 << PB3)) == 0) {
		snake_direction = LEFT;
	}
	if ((PINB & (1 << PB4)) == 0) {
		snake_direction = DOWN;
	}
}


// Generate a random point on map
coord generate_coord(void) {
    coord coordinates;
    
    coordinates.x = rand() % LCD_WIDTH;
    coordinates.y = rand() % LCD_HEIGHT;
    
    return coordinates;
}



// Initialize the snake
void initSnake(uint32_t seed) {
	int i = 0;
	snake_size = INITIAL_SIZE;
	
	while (i != INITIAL_SIZE) {
		snake[i].x = (LCD_WIDTH - INITIAL_SIZE) / 2 + i * BLOCK_SIZE;
		snake[i].y = LCD_HEIGHT / 2;
		++i;
	}
	
	head_position = INITIAL_SIZE - 1;
	tail_position = 0;
	
	srand(seed);
	food = generate_coord();
} 


// Update the game 
void updateGame(void) {

	uint32_t prev;
	prev = head_position;
	head_position = (head_position + 1) % MAX_SIZE;
	snake[head_position] = snake[prev];
	
	
	// Check snake's direction
	if (snake_direction == UP) {
		snake[head_position].y = (snake[head_position].y + BLOCK_SIZE) % LCD_HEIGHT;
	} else if (snake_direction == DOWN) {
		  snake[head_position].y = (snake[head_position].y - BLOCK_SIZE) % LCD_HEIGHT;
	} else if (snake_direction == LEFT) {
	   snake[head_position].x = (snake[head_position].x - BLOCK_SIZE) % LCD_WIDTH;
	} else if (snake_direction == RIGHT) {
		snake[head_position].x = (snake[head_position].x + BLOCK_SIZE) % LCD_WIDTH;
	}
	
	// Snake reaches the food
    if (abs(food.x - snake[head_position].x) < BLOCK_SIZE && abs(food.y - snake[head_position].y) < BLOCK_SIZE) {
        ++snake_size;
        food = generate_coord();
    } else {
        // Update snake's tail
        tail_position = (tail_position + 1) % MAX_SIZE;
    }

}

// Draw the first state of the game
void draw_start(void) {
    LCD_write_string(2, 2,"START GAME!");
}

// Draw the game
void draw_game(void) {
    uint32_t i;
    coord block;
    
    for (i = 0; i < snake_size; ++i) {
        block = snake[(tail_position + i) % MAX_SIZE];
		// TODO
		LCD_write_string(block.x + i, block.y + i, "*");
    }
    
    // draw food
	// TODO
	LCD_write_string(food.x, food.y, "F");
}


// Draw the final state of the game
void draw_end(void) {
	LCD_write_string(2, 2, "YOU WON!");
}



int main(void) 
{
	LCD_init();    
	LCD_clear();      
	init_buttons();
	snake_direction = NONE;
	

	draw_start();
	
	initSnake(42);
	
	// Wait for user input
	while(1) {
        checkInput();
		
        if (snake_direction == NONE)  
			draw_start();
		else
			break;
        _delay_ms(100);
    }
	
	// Game start announcer
	LCD_clear();
	LCD_write_string(1, 1,"STARTED!");
	_delay_ms(100);
	LCD_clear();
	
	// Snake appears
	while(snake_size < 60) {
		checkInput();
		updateGame();
		draw_game();
		_delay_ms(300);
	}
	
	// End of game
	draw_end();
	
	return 0;
}



