#include<avr/io.h>
#include <util/delay.h>
#include <stdio.h> 
#include <stdlib.h> 
#include "bmp_pixel.h"
#include "Nokia5110.h"

#define INITIAL_LENGTH 3
#define WIDTH 84
#define HEIGHT 48
#define DOWN 4
#define LEFT 3
#define UP 2
#define RIGHT 5
#define NO_DIRECTION 0
#define MAX_LENGTH 30
#define BLOCK_SIZE 6


typedef struct {
	uint8_t x;
	uint8_t y;
} pixel;

typedef struct {
	pixel body[MAX_LENGTH];
	uint8_t direction;
	uint8_t head;
	uint8_t tail;
	uint8_t size;
	uint8_t prev_direction;
}snake;

snake mySnake;

pixel generate_coordinates() {
	pixel coord;
	srand(1);
	coord.x = rand() % WIDTH;
	coord.y = rand() % HEIGHT;
	return coord;
};

void init_buttons() {
    DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);
	
	DDRB &= ~(1 << PB3);
	PORTB |= (1 << PB3);
	
	DDRB &= ~(1 << PB4);
	PORTB |= (1 << PB4);
	
	DDRB &= ~(1 << PB5);
	PORTB |= (1 << PB5);
}

void set_button() {
	if ((PINB & (1 << PB2)) == 0) {
		mySnake.direction = DOWN;
	}
	if ((PINB & (1 << PB3)) == 0) {
		mySnake.direction = LEFT;
	}
	if ((PINB & (1 << PB4)) == 0) {
		mySnake.direction = UP;
	}
	if ((PINB & (1 << PB5)) == 0) {
		mySnake.direction = RIGHT;
	}
}

void init_snake() {
	mySnake.body[0].x = 54;
	mySnake.body[0].y = 18;
	mySnake.body[1].x = mySnake.body[0].x + BLOCK_SIZE;
	mySnake.body[1].y = mySnake.body[0].y;
	mySnake.body[2].x = mySnake.body[1].x + BLOCK_SIZE;
	mySnake.body[2].y = mySnake.body[1].y;
	mySnake.size = INITIAL_LENGTH;
	mySnake.head = 0;
	mySnake.tail = 2;
	mySnake.direction = NO_DIRECTION;
	mySnake.prev_direction = NO_DIRECTION;
}

void update() {
	pixel food;
	food = generate_coordinates();
	pixel prev_position = mySnake.body[0];
	pixel aux;
	uint8_t i;
	
	if((mySnake.direction == RIGHT && mySnake.prev_direction != LEFT) ||
	(mySnake.direction == LEFT && mySnake.prev_direction != RIGHT) ||
	(mySnake.direction == UP && mySnake.prev_direction != DOWN) ||
	(mySnake.direction == DOWN && mySnake.prev_direction != UP)) {
		if(mySnake.direction == LEFT) {
			mySnake.prev_direction = mySnake.direction;
			i = 0;
			while(i < mySnake.size) {
				if(i == 0){
					mySnake.body[i].x = mySnake.body[i].x - BLOCK_SIZE;
				} else {
					aux = mySnake.body[i];
					mySnake.body[i] = prev_position;
					prev_position = aux;
				}
				i++;
			}
			mySnake.direction = NO_DIRECTION;
		}
		if(mySnake.direction == DOWN) {
			mySnake.prev_direction = mySnake.direction;
			i = 0;
			while(i < mySnake.size) {
				if(i == 0){
					mySnake.body[i].y = mySnake.body[i].y - 1;
				} else {
					aux = mySnake.body[i];
					mySnake.body[i] = prev_position;
					prev_position = aux;
				}
				i++;
			}
			mySnake.direction = NO_DIRECTION;
		}
		if(mySnake.direction == UP) {
			mySnake.prev_direction = mySnake.direction;
			i = 0;
			while(i < mySnake.size) {
				if(i == 0){
					mySnake.body[i].y = mySnake.body[i].y + 1;
				} else {
					aux = mySnake.body[i];
					mySnake.body[i] = prev_position;
					prev_position = aux;
				}
				i++;
			}
			mySnake.direction = NO_DIRECTION;
		}
		if(mySnake.direction == RIGHT) {
			mySnake.prev_direction = mySnake.direction;
			i = 0;
			while(i < mySnake.size) {
				if(i == 0){
					mySnake.body[i].x = mySnake.body[i].x + BLOCK_SIZE;
				} else {
					aux = mySnake.body[i];
					mySnake.body[i] = prev_position;
					prev_position = aux;
				}
				i++;
			}
			mySnake.direction = NO_DIRECTION;
		}
	}
	LCD_write_char('&');
}

void draw_snake() {
	uint8_t i;
	
	for(i = 0; i < mySnake.size; i++) {
		if(i == 0)
			LCD_write_string(mySnake.body[i].x, mySnake.body[i].y, "+");
		else
			LCD_write_string(mySnake.body[i].x, mySnake.body[i].y, "*");
	}
}

void start_frame() {
	LCD_write_string(1, 1, "Press any key");
	LCD_write_string(15, 2, "to start");
	LCD_write_string(15, 3, "the game!");
}

void write_button(){
		if(mySnake.direction == NO_DIRECTION)
			LCD_write_string(2,2, "No button");
		if(mySnake.direction == DOWN)
			LCD_write_string(2,2, "DOWN");
		if(mySnake.direction == LEFT)
			LCD_write_string(2,2, "LEFT");
		if(mySnake.direction == UP)
			LCD_write_string(2,2, "UP");
		if(mySnake.direction == RIGHT)
			LCD_write_string(2,2, "RIGHT");
}

int main() {
	
	LCD_init();
	LCD_clear();
	
	init_buttons();
	start_frame();
	_delay_ms(500);
	LCD_clear();
	
	init_snake();
	start_frame();
	_delay_ms(7000);
	draw_snake();
	while(1) {
		set_button();
		LCD_clear();
		update();
		draw_snake();
		_delay_ms(100);
	}
	
	return 0;
}